# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.phone_numbers import (
    job_list_params,
    job_delete_batch_params,
    job_update_batch_params,
    job_update_emergency_settings_batch_params,
)
from ...types.phone_numbers.phone_numbers_job import PhoneNumbersJob
from ...types.phone_numbers.job_retrieve_response import JobRetrieveResponse
from ...types.phone_numbers.job_delete_batch_response import JobDeleteBatchResponse
from ...types.phone_numbers.job_update_batch_response import JobUpdateBatchResponse
from ...types.phone_numbers.update_voice_settings_param import UpdateVoiceSettingsParam
from ...types.phone_numbers.job_update_emergency_settings_batch_response import JobUpdateEmergencySettingsBatchResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobRetrieveResponse:
        """
        Retrieve a phone numbers job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/phone_numbers/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: job_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PhoneNumbersJob]:
        """
        Lists the phone numbers jobs

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[type]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/phone_numbers/jobs",
            page=SyncDefaultFlatPagination[PhoneNumbersJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=PhoneNumbersJob,
        )

    def delete_batch(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobDeleteBatchResponse:
        """Creates a new background job to delete a batch of numbers.

        At most one thousand
        numbers can be updated per API call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/phone_numbers/jobs/delete_phone_numbers",
            body=maybe_transform({"phone_numbers": phone_numbers}, job_delete_batch_params.JobDeleteBatchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobDeleteBatchResponse,
        )

    def update_batch(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        filter: job_update_batch_params.Filter | Omit = omit,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        deletion_lock_enabled: bool | Omit = omit,
        external_pin: str | Omit = omit,
        hd_voice_enabled: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        voice: UpdateVoiceSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobUpdateBatchResponse:
        """Creates a new background job to update a batch of numbers.

        At most one thousand
        numbers can be updated per API call. At least one of the updateable fields must
        be submitted. IMPORTANT: You must either specify filters (using the filter
        parameters) or specific phone numbers (using the phone_numbers parameter in the
        request body). If you specify filters, ALL phone numbers that match the given
        filters (up to 1000 at a time) will be updated. If you want to update only
        specific numbers, you must use the phone_numbers parameter in the request body.
        When using the phone_numbers parameter, ensure you follow the correct format as
        shown in the example (either phone number IDs or phone numbers in E164 format).

        Args:
          phone_numbers: Array of phone number ids and/or phone numbers in E164 format to update. This
              parameter is required if no filter parameters are provided. If you want to
              update specific numbers rather than all numbers matching a filter, you must use
              this parameter. Each item must be either a valid phone number ID or a phone
              number in E164 format (e.g., '+13127367254').

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[has_bundle], filter[tag], filter[connection_id], filter[phone_number],
              filter[status], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference]

          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with the phone number.

          customer_reference: A customer reference string for customer look ups.

          deletion_lock_enabled: Indicates whether to enable or disable the deletion lock on each phone number.
              When enabled, this prevents the phone number from being deleted via the API or
              Telnyx portal.

          external_pin: If someone attempts to port your phone number away from Telnyx and your phone
              number has an external PIN set, we will attempt to verify that you provided the
              correct external PIN to the winning carrier. Note that not all carriers
              cooperate with this security mechanism.

          hd_voice_enabled: Indicates whether to enable or disable HD Voice on each phone number. HD Voice
              is a paid feature and may not be available for all phone numbers, more details
              about it can be found in the Telnyx support documentation.

          tags: A list of user-assigned tags to help organize phone numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/phone_numbers/jobs/update_phone_numbers",
            body=maybe_transform(
                {
                    "phone_numbers": phone_numbers,
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "deletion_lock_enabled": deletion_lock_enabled,
                    "external_pin": external_pin,
                    "hd_voice_enabled": hd_voice_enabled,
                    "tags": tags,
                    "voice": voice,
                },
                job_update_batch_params.JobUpdateBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, job_update_batch_params.JobUpdateBatchParams),
            ),
            cast_to=JobUpdateBatchResponse,
        )

    def update_emergency_settings_batch(
        self,
        *,
        emergency_enabled: bool,
        phone_numbers: SequenceNotStr[str],
        emergency_address_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobUpdateEmergencySettingsBatchResponse:
        """
        Creates a background job to update the emergency settings of a collection of
        phone numbers. At most one thousand numbers can be updated per API call.

        Args:
          emergency_enabled: Indicates whether to enable or disable emergency services on the numbers.

          emergency_address_id: Identifies the address to be used with emergency services. Required if
              emergency_enabled is true, must be null or omitted if emergency_enabled is
              false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/phone_numbers/jobs/update_emergency_settings",
            body=maybe_transform(
                {
                    "emergency_enabled": emergency_enabled,
                    "phone_numbers": phone_numbers,
                    "emergency_address_id": emergency_address_id,
                },
                job_update_emergency_settings_batch_params.JobUpdateEmergencySettingsBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobUpdateEmergencySettingsBatchResponse,
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobRetrieveResponse:
        """
        Retrieve a phone numbers job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/phone_numbers/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: job_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumbersJob, AsyncDefaultFlatPagination[PhoneNumbersJob]]:
        """
        Lists the phone numbers jobs

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[type]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/phone_numbers/jobs",
            page=AsyncDefaultFlatPagination[PhoneNumbersJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=PhoneNumbersJob,
        )

    async def delete_batch(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobDeleteBatchResponse:
        """Creates a new background job to delete a batch of numbers.

        At most one thousand
        numbers can be updated per API call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/phone_numbers/jobs/delete_phone_numbers",
            body=await async_maybe_transform(
                {"phone_numbers": phone_numbers}, job_delete_batch_params.JobDeleteBatchParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobDeleteBatchResponse,
        )

    async def update_batch(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        filter: job_update_batch_params.Filter | Omit = omit,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        deletion_lock_enabled: bool | Omit = omit,
        external_pin: str | Omit = omit,
        hd_voice_enabled: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        voice: UpdateVoiceSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobUpdateBatchResponse:
        """Creates a new background job to update a batch of numbers.

        At most one thousand
        numbers can be updated per API call. At least one of the updateable fields must
        be submitted. IMPORTANT: You must either specify filters (using the filter
        parameters) or specific phone numbers (using the phone_numbers parameter in the
        request body). If you specify filters, ALL phone numbers that match the given
        filters (up to 1000 at a time) will be updated. If you want to update only
        specific numbers, you must use the phone_numbers parameter in the request body.
        When using the phone_numbers parameter, ensure you follow the correct format as
        shown in the example (either phone number IDs or phone numbers in E164 format).

        Args:
          phone_numbers: Array of phone number ids and/or phone numbers in E164 format to update. This
              parameter is required if no filter parameters are provided. If you want to
              update specific numbers rather than all numbers matching a filter, you must use
              this parameter. Each item must be either a valid phone number ID or a phone
              number in E164 format (e.g., '+13127367254').

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[has_bundle], filter[tag], filter[connection_id], filter[phone_number],
              filter[status], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference]

          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with the phone number.

          customer_reference: A customer reference string for customer look ups.

          deletion_lock_enabled: Indicates whether to enable or disable the deletion lock on each phone number.
              When enabled, this prevents the phone number from being deleted via the API or
              Telnyx portal.

          external_pin: If someone attempts to port your phone number away from Telnyx and your phone
              number has an external PIN set, we will attempt to verify that you provided the
              correct external PIN to the winning carrier. Note that not all carriers
              cooperate with this security mechanism.

          hd_voice_enabled: Indicates whether to enable or disable HD Voice on each phone number. HD Voice
              is a paid feature and may not be available for all phone numbers, more details
              about it can be found in the Telnyx support documentation.

          tags: A list of user-assigned tags to help organize phone numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/phone_numbers/jobs/update_phone_numbers",
            body=await async_maybe_transform(
                {
                    "phone_numbers": phone_numbers,
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "deletion_lock_enabled": deletion_lock_enabled,
                    "external_pin": external_pin,
                    "hd_voice_enabled": hd_voice_enabled,
                    "tags": tags,
                    "voice": voice,
                },
                job_update_batch_params.JobUpdateBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"filter": filter}, job_update_batch_params.JobUpdateBatchParams),
            ),
            cast_to=JobUpdateBatchResponse,
        )

    async def update_emergency_settings_batch(
        self,
        *,
        emergency_enabled: bool,
        phone_numbers: SequenceNotStr[str],
        emergency_address_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobUpdateEmergencySettingsBatchResponse:
        """
        Creates a background job to update the emergency settings of a collection of
        phone numbers. At most one thousand numbers can be updated per API call.

        Args:
          emergency_enabled: Indicates whether to enable or disable emergency services on the numbers.

          emergency_address_id: Identifies the address to be used with emergency services. Required if
              emergency_enabled is true, must be null or omitted if emergency_enabled is
              false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/phone_numbers/jobs/update_emergency_settings",
            body=await async_maybe_transform(
                {
                    "emergency_enabled": emergency_enabled,
                    "phone_numbers": phone_numbers,
                    "emergency_address_id": emergency_address_id,
                },
                job_update_emergency_settings_batch_params.JobUpdateEmergencySettingsBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobUpdateEmergencySettingsBatchResponse,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.delete_batch = to_raw_response_wrapper(
            jobs.delete_batch,
        )
        self.update_batch = to_raw_response_wrapper(
            jobs.update_batch,
        )
        self.update_emergency_settings_batch = to_raw_response_wrapper(
            jobs.update_emergency_settings_batch,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.delete_batch = async_to_raw_response_wrapper(
            jobs.delete_batch,
        )
        self.update_batch = async_to_raw_response_wrapper(
            jobs.update_batch,
        )
        self.update_emergency_settings_batch = async_to_raw_response_wrapper(
            jobs.update_emergency_settings_batch,
        )


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.delete_batch = to_streamed_response_wrapper(
            jobs.delete_batch,
        )
        self.update_batch = to_streamed_response_wrapper(
            jobs.update_batch,
        )
        self.update_emergency_settings_batch = to_streamed_response_wrapper(
            jobs.update_emergency_settings_batch,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.delete_batch = async_to_streamed_response_wrapper(
            jobs.delete_batch,
        )
        self.update_batch = async_to_streamed_response_wrapper(
            jobs.update_batch,
        )
        self.update_emergency_settings_batch = async_to_streamed_response_wrapper(
            jobs.update_emergency_settings_batch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .room import Room
from .._models import BaseModel

__all__ = ["RoomUpdateResponse"]


class RoomUpdateResponse(BaseModel):
    data: Optional[Room] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import AccessIPAddressResponse
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccessIPAddress:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.create(
            ip_address="ip_address",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.create(
            ip_address="ip_address",
            description="description",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.access_ip_address.with_raw_response.create(
            ip_address="ip_address",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.access_ip_address.with_streaming_response.create(
            ip_address="ip_address",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.retrieve(
            "access_ip_address_id",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.access_ip_address.with_raw_response.retrieve(
            "access_ip_address_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.access_ip_address.with_streaming_response.retrieve(
            "access_ip_address_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `access_ip_address_id` but received ''"):
            client.access_ip_address.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.list()
        assert_matches_type(SyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.list(
            filter={
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "ip_address": "ip_address",
                "ip_source": "ip_source",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.access_ip_address.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.access_ip_address.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        access_ip_address = client.access_ip_address.delete(
            "access_ip_address_id",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.access_ip_address.with_raw_response.delete(
            "access_ip_address_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.access_ip_address.with_streaming_response.delete(
            "access_ip_address_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `access_ip_address_id` but received ''"):
            client.access_ip_address.with_raw_response.delete(
                "",
            )


class TestAsyncAccessIPAddress:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.create(
            ip_address="ip_address",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.create(
            ip_address="ip_address",
            description="description",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.access_ip_address.with_raw_response.create(
            ip_address="ip_address",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = await response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.access_ip_address.with_streaming_response.create(
            ip_address="ip_address",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = await response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.retrieve(
            "access_ip_address_id",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.access_ip_address.with_raw_response.retrieve(
            "access_ip_address_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = await response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.access_ip_address.with_streaming_response.retrieve(
            "access_ip_address_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = await response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `access_ip_address_id` but received ''"):
            await async_client.access_ip_address.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.list()
        assert_matches_type(AsyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.list(
            filter={
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "ip_address": "ip_address",
                "ip_source": "ip_source",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.access_ip_address.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.access_ip_address.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[AccessIPAddressResponse], access_ip_address, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        access_ip_address = await async_client.access_ip_address.delete(
            "access_ip_address_id",
        )
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.access_ip_address.with_raw_response.delete(
            "access_ip_address_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        access_ip_address = await response.parse()
        assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.access_ip_address.with_streaming_response.delete(
            "access_ip_address_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            access_ip_address = await response.parse()
            assert_matches_type(AccessIPAddressResponse, access_ip_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `access_ip_address_id` but received ''"):
            await async_client.access_ip_address.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .inference_embedding_bucket_ids_param import InferenceEmbeddingBucketIDsParam

__all__ = [
    "ChatCreateCompletionParams",
    "Message",
    "MessageContentTextAndImageArray",
    "ResponseFormat",
    "Tool",
    "ToolFunction",
    "ToolFunctionFunction",
    "ToolRetrieval",
]


class ChatCreateCompletionParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """A list of the previous chat messages for context."""

    api_key_ref: str
    """
    If you are using an external inference provider like xAI or OpenAI, this field
    allows you to pass along a reference to your API key. After creating an
    [integration secret](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    for you API key, pass the secret's `identifier` in this field.
    """

    best_of: int
    """
    This is used with `use_beam_search` to determine how many candidate beams to
    explore.
    """

    early_stopping: bool
    """This is used with `use_beam_search`.

    If `true`, generation stops as soon as there are `best_of` complete candidates;
    if `false`, a heuristic is applied and the generation stops when is it very
    unlikely to find better candidates.
    """

    frequency_penalty: float
    """Higher values will penalize the model from repeating the same output tokens."""

    guided_choice: SequenceNotStr[str]
    """If specified, the output will be exactly one of the choices."""

    guided_json: Dict[str, object]
    """Must be a valid JSON schema.

    If specified, the output will follow the JSON schema.
    """

    guided_regex: str
    """If specified, the output will follow the regex pattern."""

    length_penalty: float
    """This is used with `use_beam_search` to prefer shorter or longer completions."""

    logprobs: bool
    """Whether to return log probabilities of the output tokens or not.

    If true, returns the log probabilities of each output token returned in the
    `content` of `message`.
    """

    max_tokens: int
    """Maximum number of completion tokens the model should generate."""

    min_p: float
    """
    This is an alternative to `top_p` that
    [many prefer](https://github.com/huggingface/transformers/issues/27670). Must be
    in [0, 1].
    """

    model: str
    """The language model to chat with."""

    n: float
    """This will return multiple choices for you instead of a single chat completion."""

    presence_penalty: float
    """Higher values will penalize the model from repeating the same output tokens."""

    response_format: ResponseFormat
    """Use this is you want to guarantee a JSON output without defining a schema.

    For control over the schema, use `guided_json`.
    """

    stream: bool
    """Whether or not to stream data-only server-sent events as they become available."""

    temperature: float
    """Adjusts the "creativity" of the model.

    Lower values make the model more deterministic and repetitive, while higher
    values make the model more random and creative.
    """

    tool_choice: Literal["none", "auto", "required"]

    tools: Iterable[Tool]
    """
    The `function` tool type follows the same schema as the
    [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat).
    The `retrieval` tool type is unique to Telnyx. You may pass a list of
    [embedded storage buckets](https://developers.telnyx.com/api-reference/embeddings/embed-documents)
    for retrieval-augmented generation.
    """

    top_logprobs: int
    """This is used with `logprobs`.

    An integer between 0 and 20 specifying the number of most likely tokens to
    return at each token position, each with an associated log probability.
    """

    top_p: float
    """An alternative or complement to `temperature`.

    This adjusts how many of the top possibilities to consider.
    """

    use_beam_search: bool
    """
    Setting this to `true` will allow the model to
    [explore more completion options](https://huggingface.co/blog/how-to-generate#beam-search).
    This is not supported by OpenAI.
    """


class MessageContentTextAndImageArray(TypedDict, total=False):
    type: Required[Literal["text", "image_url"]]

    image_url: str

    text: str


class Message(TypedDict, total=False):
    content: Required[Union[str, Iterable[MessageContentTextAndImageArray]]]

    role: Required[Literal["system", "user", "assistant", "tool"]]


class ResponseFormat(TypedDict, total=False):
    """Use this is you want to guarantee a JSON output without defining a schema.

    For control over the schema, use `guided_json`.
    """

    type: Required[Literal["text", "json_object"]]


class ToolFunctionFunction(TypedDict, total=False):
    name: Required[str]

    description: str

    parameters: Dict[str, object]


class ToolFunction(TypedDict, total=False):
    function: Required[ToolFunctionFunction]

    type: Required[Literal["function"]]


class ToolRetrieval(TypedDict, total=False):
    retrieval: Required[InferenceEmbeddingBucketIDsParam]

    type: Required[Literal["retrieval"]]


Tool: TypeAlias = Union[ToolFunction, ToolRetrieval]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .fax_failed_webhook_event import FaxFailedWebhookEvent
from .fax_queued_webhook_event import FaxQueuedWebhookEvent
from .call_hangup_webhook_event import CallHangupWebhookEvent
from .call_bridged_webhook_event import CallBridgedWebhookEvent
from .call_answered_webhook_event import CallAnsweredWebhookEvent
from .call_enqueued_webhook_event import CallEnqueuedWebhookEvent
from .fax_delivered_webhook_event import FaxDeliveredWebhookEvent
from .transcription_webhook_event import TranscriptionWebhookEvent
from .call_initiated_webhook_event import CallInitiatedWebhookEvent
from .call_left_queue_webhook_event import CallLeftQueueWebhookEvent
from .delivery_update_webhook_event import DeliveryUpdateWebhookEvent
from .inbound_message_webhook_event import InboundMessageWebhookEvent
from .call_speak_ended_webhook_event import CallSpeakEndedWebhookEvent
from .conference_ended_webhook_event import ConferenceEndedWebhookEvent
from .call_fork_started_webhook_event import CallForkStartedWebhookEvent
from .call_fork_stopped_webhook_event import CallForkStoppedWebhookEvent
from .call_gather_ended_webhook_event import CallGatherEndedWebhookEvent
from .call_refer_failed_webhook_event import CallReferFailedWebhookEvent
from .call_dtmf_received_webhook_event import CallDtmfReceivedWebhookEvent
from .call_refer_started_webhook_event import CallReferStartedWebhookEvent
from .call_siprec_failed_webhook_event import CallSiprecFailedWebhookEvent
from .call_speak_started_webhook_event import CallSpeakStartedWebhookEvent
from .conference_created_webhook_event import ConferenceCreatedWebhookEvent
from .call_playback_ended_webhook_event import CallPlaybackEndedWebhookEvent
from .call_siprec_started_webhook_event import CallSiprecStartedWebhookEvent
from .call_siprec_stopped_webhook_event import CallSiprecStoppedWebhookEvent
from .fax_media_processed_webhook_event import FaxMediaProcessedWebhookEvent
from .fax_sending_started_webhook_event import FaxSendingStartedWebhookEvent
from .replaced_link_click_webhook_event import ReplacedLinkClickWebhookEvent
from .call_ai_gather_ended_webhook_event import CallAIGatherEndedWebhookEvent
from .call_recording_error_webhook_event import CallRecordingErrorWebhookEvent
from .call_recording_saved_webhook_event import CallRecordingSavedWebhookEvent
from .call_refer_completed_webhook_event import CallReferCompletedWebhookEvent
from .call_playback_started_webhook_event import CallPlaybackStartedWebhookEvent
from .call_streaming_failed_webhook_event import CallStreamingFailedWebhookEvent
from .call_streaming_started_webhook_event import CallStreamingStartedWebhookEvent
from .call_streaming_stopped_webhook_event import CallStreamingStoppedWebhookEvent
from .campaign_status_update_webhook_event import CampaignStatusUpdateWebhookEvent
from .conference_speak_ended_webhook_event import ConferenceSpeakEndedWebhookEvent
from .call_conversation_ended_webhook_event import CallConversationEndedWebhookEvent
from .conference_floor_changed_webhook_event import ConferenceFloorChangedWebhookEvent
from .conference_speak_started_webhook_event import ConferenceSpeakStartedWebhookEvent
from .conference_playback_ended_webhook_event import ConferencePlaybackEndedWebhookEvent
from .conference_recording_saved_webhook_event import ConferenceRecordingSavedWebhookEvent
from .number_order_status_update_webhook_event import NumberOrderStatusUpdateWebhookEvent
from .call_machine_greeting_ended_webhook_event import CallMachineGreetingEndedWebhookEvent
from .conference_participant_left_webhook_event import ConferenceParticipantLeftWebhookEvent
from .conference_playback_started_webhook_event import ConferencePlaybackStartedWebhookEvent
from .call_machine_detection_ended_webhook_event import CallMachineDetectionEndedWebhookEvent
from .conference_participant_joined_webhook_event import ConferenceParticipantJoinedWebhookEvent
from .call_ai_gather_partial_results_webhook_event import CallAIGatherPartialResultsWebhookEvent
from .call_recording_transcription_saved_webhook_event import CallRecordingTranscriptionSavedWebhookEvent
from .conference_participant_speak_ended_webhook_event import ConferenceParticipantSpeakEndedWebhookEvent
from .call_machine_premium_greeting_ended_webhook_event import CallMachinePremiumGreetingEndedWebhookEvent
from .call_conversation_insights_generated_webhook_event import CallConversationInsightsGeneratedWebhookEvent
from .call_machine_premium_detection_ended_webhook_event import CallMachinePremiumDetectionEndedWebhookEvent
from .conference_participant_speak_started_webhook_event import ConferenceParticipantSpeakStartedWebhookEvent
from .conference_participant_playback_ended_webhook_event import ConferenceParticipantPlaybackEndedWebhookEvent
from .call_ai_gather_message_history_updated_webhook_event import CallAIGatherMessageHistoryUpdatedWebhookEvent
from .conference_participant_playback_started_webhook_event import ConferenceParticipantPlaybackStartedWebhookEvent

__all__ = ["UnwrapWebhookEvent"]

UnwrapWebhookEvent: TypeAlias = Union[
    CallAIGatherEndedWebhookEvent,
    CallAIGatherMessageHistoryUpdatedWebhookEvent,
    CallAIGatherPartialResultsWebhookEvent,
    CallAnsweredWebhookEvent,
    CallBridgedWebhookEvent,
    CallConversationEndedWebhookEvent,
    CallConversationInsightsGeneratedWebhookEvent,
    CallDtmfReceivedWebhookEvent,
    CallEnqueuedWebhookEvent,
    CallForkStartedWebhookEvent,
    CallForkStoppedWebhookEvent,
    CallGatherEndedWebhookEvent,
    CallHangupWebhookEvent,
    CallInitiatedWebhookEvent,
    CallLeftQueueWebhookEvent,
    CallMachineDetectionEndedWebhookEvent,
    CallMachineGreetingEndedWebhookEvent,
    CallMachinePremiumDetectionEndedWebhookEvent,
    CallMachinePremiumGreetingEndedWebhookEvent,
    CallPlaybackEndedWebhookEvent,
    CallPlaybackStartedWebhookEvent,
    CallRecordingErrorWebhookEvent,
    CallRecordingSavedWebhookEvent,
    CallRecordingTranscriptionSavedWebhookEvent,
    CallReferCompletedWebhookEvent,
    CallReferFailedWebhookEvent,
    CallReferStartedWebhookEvent,
    CallSiprecFailedWebhookEvent,
    CallSiprecStartedWebhookEvent,
    CallSiprecStoppedWebhookEvent,
    CallSpeakEndedWebhookEvent,
    CallSpeakStartedWebhookEvent,
    CallStreamingFailedWebhookEvent,
    CallStreamingStartedWebhookEvent,
    CallStreamingStoppedWebhookEvent,
    CampaignStatusUpdateWebhookEvent,
    ConferenceCreatedWebhookEvent,
    ConferenceEndedWebhookEvent,
    ConferenceFloorChangedWebhookEvent,
    ConferenceParticipantJoinedWebhookEvent,
    ConferenceParticipantLeftWebhookEvent,
    ConferenceParticipantPlaybackEndedWebhookEvent,
    ConferenceParticipantPlaybackStartedWebhookEvent,
    ConferenceParticipantSpeakEndedWebhookEvent,
    ConferenceParticipantSpeakStartedWebhookEvent,
    ConferencePlaybackEndedWebhookEvent,
    ConferencePlaybackStartedWebhookEvent,
    ConferenceRecordingSavedWebhookEvent,
    ConferenceSpeakEndedWebhookEvent,
    ConferenceSpeakStartedWebhookEvent,
    DeliveryUpdateWebhookEvent,
    FaxDeliveredWebhookEvent,
    FaxFailedWebhookEvent,
    FaxMediaProcessedWebhookEvent,
    FaxQueuedWebhookEvent,
    FaxSendingStartedWebhookEvent,
    InboundMessageWebhookEvent,
    NumberOrderStatusUpdateWebhookEvent,
    ReplacedLinkClickWebhookEvent,
    TranscriptionWebhookEvent,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.ai import (
    AssistantsList,
    InferenceEmbedding,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVersions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
            include_mcp_servers=True,
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.assistants.versions.with_raw_response.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.assistants.versions.with_streaming_response.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.retrieve(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.retrieve(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.update(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.update(
            version_id="version_id",
            assistant_id="assistant_id",
            description="description",
            dynamic_variables={"foo": "bar"},
            dynamic_variables_webhook_url="dynamic_variables_webhook_url",
            enabled_features=["telephony"],
            greeting="greeting",
            insight_settings={"insight_group_id": "insight_group_id"},
            instructions="instructions",
            llm_api_key_ref="llm_api_key_ref",
            messaging_settings={
                "default_messaging_profile_id": "default_messaging_profile_id",
                "delivery_status_webhook_url": "delivery_status_webhook_url",
            },
            model="model",
            name="name",
            privacy_settings={"data_retention": True},
            telephony_settings={
                "default_texml_app_id": "default_texml_app_id",
                "noise_suppression": "deepfilternet",
                "noise_suppression_config": {
                    "attenuation_limit": 0,
                    "mode": "advanced",
                },
                "supports_unauthenticated_web_calls": True,
                "time_limit_secs": 30,
            },
            tools=[
                {
                    "type": "webhook",
                    "webhook": {
                        "description": "description",
                        "name": "name",
                        "url": "https://example.com/api/v1/function",
                        "body_parameters": {
                            "properties": {
                                "age": "bar",
                                "location": "bar",
                            },
                            "required": ["age", "location"],
                            "type": "object",
                        },
                        "headers": [
                            {
                                "name": "name",
                                "value": "value",
                            }
                        ],
                        "method": "GET",
                        "path_parameters": {
                            "properties": {"id": "bar"},
                            "required": ["id"],
                            "type": "object",
                        },
                        "query_parameters": {
                            "properties": {"page": "bar"},
                            "required": ["page"],
                            "type": "object",
                        },
                    },
                }
            ],
            transcription={
                "language": "language",
                "model": "deepgram/flux",
                "region": "region",
                "settings": {
                    "eager_eot_threshold": 0.3,
                    "eot_threshold": 0,
                    "eot_timeout_ms": 0,
                    "numerals": True,
                    "smart_format": True,
                },
            },
            voice_settings={
                "voice": "voice",
                "api_key_ref": "api_key_ref",
                "background_audio": {
                    "type": "predefined_media",
                    "value": "silence",
                },
                "similarity_boost": 0,
                "speed": 0,
                "style": 0,
                "temperature": 0,
                "use_speaker_boost": True,
                "voice_speed": 0,
            },
            widget_settings={
                "agent_thinking_text": "agent_thinking_text",
                "audio_visualizer_config": {
                    "color": "verdant",
                    "preset": "preset",
                },
                "default_state": "expanded",
                "give_feedback_url": "give_feedback_url",
                "logo_icon_url": "logo_icon_url",
                "position": "fixed",
                "report_issue_url": "report_issue_url",
                "speak_to_interrupt_text": "speak_to_interrupt_text",
                "start_call_text": "start_call_text",
                "theme": "light",
                "view_history_url": "view_history_url",
            },
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.ai.assistants.versions.with_raw_response.update(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.ai.assistants.versions.with_streaming_response.update(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.update(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.update(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.list(
            "assistant_id",
        )
        assert_matches_type(AssistantsList, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.assistants.versions.with_raw_response.list(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(AssistantsList, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.assistants.versions.with_streaming_response.list(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert_matches_type(AssistantsList, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert version is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ai.assistants.versions.with_raw_response.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert version is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ai.assistants.versions.with_streaming_response.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert version is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.delete(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.delete(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_promote(self, client: Telnyx) -> None:
        version = client.ai.assistants.versions.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_promote(self, client: Telnyx) -> None:
        response = client.ai.assistants.versions.with_raw_response.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_promote(self, client: Telnyx) -> None:
        with client.ai.assistants.versions.with_streaming_response.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_promote(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.promote(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            client.ai.assistants.versions.with_raw_response.promote(
                version_id="",
                assistant_id="assistant_id",
            )


class TestAsyncVersions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
            include_mcp_servers=True,
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.versions.with_raw_response.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = await response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.versions.with_streaming_response.retrieve(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.retrieve(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.retrieve(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.update(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.update(
            version_id="version_id",
            assistant_id="assistant_id",
            description="description",
            dynamic_variables={"foo": "bar"},
            dynamic_variables_webhook_url="dynamic_variables_webhook_url",
            enabled_features=["telephony"],
            greeting="greeting",
            insight_settings={"insight_group_id": "insight_group_id"},
            instructions="instructions",
            llm_api_key_ref="llm_api_key_ref",
            messaging_settings={
                "default_messaging_profile_id": "default_messaging_profile_id",
                "delivery_status_webhook_url": "delivery_status_webhook_url",
            },
            model="model",
            name="name",
            privacy_settings={"data_retention": True},
            telephony_settings={
                "default_texml_app_id": "default_texml_app_id",
                "noise_suppression": "deepfilternet",
                "noise_suppression_config": {
                    "attenuation_limit": 0,
                    "mode": "advanced",
                },
                "supports_unauthenticated_web_calls": True,
                "time_limit_secs": 30,
            },
            tools=[
                {
                    "type": "webhook",
                    "webhook": {
                        "description": "description",
                        "name": "name",
                        "url": "https://example.com/api/v1/function",
                        "body_parameters": {
                            "properties": {
                                "age": "bar",
                                "location": "bar",
                            },
                            "required": ["age", "location"],
                            "type": "object",
                        },
                        "headers": [
                            {
                                "name": "name",
                                "value": "value",
                            }
                        ],
                        "method": "GET",
                        "path_parameters": {
                            "properties": {"id": "bar"},
                            "required": ["id"],
                            "type": "object",
                        },
                        "query_parameters": {
                            "properties": {"page": "bar"},
                            "required": ["page"],
                            "type": "object",
                        },
                    },
                }
            ],
            transcription={
                "language": "language",
                "model": "deepgram/flux",
                "region": "region",
                "settings": {
                    "eager_eot_threshold": 0.3,
                    "eot_threshold": 0,
                    "eot_timeout_ms": 0,
                    "numerals": True,
                    "smart_format": True,
                },
            },
            voice_settings={
                "voice": "voice",
                "api_key_ref": "api_key_ref",
                "background_audio": {
                    "type": "predefined_media",
                    "value": "silence",
                },
                "similarity_boost": 0,
                "speed": 0,
                "style": 0,
                "temperature": 0,
                "use_speaker_boost": True,
                "voice_speed": 0,
            },
            widget_settings={
                "agent_thinking_text": "agent_thinking_text",
                "audio_visualizer_config": {
                    "color": "verdant",
                    "preset": "preset",
                },
                "default_state": "expanded",
                "give_feedback_url": "give_feedback_url",
                "logo_icon_url": "logo_icon_url",
                "position": "fixed",
                "report_issue_url": "report_issue_url",
                "speak_to_interrupt_text": "speak_to_interrupt_text",
                "start_call_text": "start_call_text",
                "theme": "light",
                "view_history_url": "view_history_url",
            },
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.versions.with_raw_response.update(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = await response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.versions.with_streaming_response.update(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.update(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.update(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.list(
            "assistant_id",
        )
        assert_matches_type(AssistantsList, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.versions.with_raw_response.list(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = await response.parse()
        assert_matches_type(AssistantsList, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.versions.with_streaming_response.list(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert_matches_type(AssistantsList, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert version is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.versions.with_raw_response.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = await response.parse()
        assert version is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.versions.with_streaming_response.delete(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert version is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.delete(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.delete(
                version_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_promote(self, async_client: AsyncTelnyx) -> None:
        version = await async_client.ai.assistants.versions.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_promote(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.versions.with_raw_response.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = await response.parse()
        assert_matches_type(InferenceEmbedding, version, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_promote(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.versions.with_streaming_response.promote(
            version_id="version_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert_matches_type(InferenceEmbedding, version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_promote(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.promote(
                version_id="version_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version_id` but received ''"):
            await async_client.ai.assistants.versions.with_raw_response.promote(
                version_id="",
                assistant_id="assistant_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo
from ..filter_param import FilterParam

__all__ = ["MessagingCreateParams"]


class MessagingCreateParams(TypedDict, total=False):
    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """End time in ISO format.

    Note: If end time includes the last 4 hours, some MDRs might not appear in this
    report, due to wait time for downstream message delivery confirmation
    """

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Start time in ISO format"""

    connections: Iterable[int]
    """List of connections to filter by"""

    directions: Iterable[int]
    """List of directions to filter by (Inbound = 1, Outbound = 2)"""

    filters: Iterable[FilterParam]
    """List of filters to apply"""

    include_message_body: bool
    """Whether to include message body in the report"""

    managed_accounts: SequenceNotStr[str]
    """List of managed accounts to include"""

    profiles: SequenceNotStr[str]
    """List of messaging profile IDs to filter by"""

    record_types: Iterable[int]
    """List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)"""

    report_name: str
    """Name of the report"""

    select_all_managed_accounts: bool
    """Whether to select all managed accounts"""

    timezone: str
    """Timezone for the report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["ConferenceRetrieveRecordingsResponse", "Recording"]


class Recording(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    call_sid: Optional[str] = None
    """The identifier of the related participant's call."""

    channels: Optional[int] = None
    """The number of channels in the recording."""

    conference_sid: Optional[str] = None
    """The identifier of the related conference."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    duration: Optional[int] = None
    """Duratin of the recording in seconds."""

    error_code: Optional[str] = None
    """The recording error, if any."""

    media_url: Optional[str] = None
    """The URL to use to download the recording."""

    sid: Optional[str] = None
    """The unique identifier of the recording."""

    source: Optional[
        Literal[
            "DialVerb",
            "Conference",
            "OutboundAPI",
            "Trunking",
            "RecordVerb",
            "StartCallRecordingAPI",
            "StartConferenceRecordingAPI",
        ]
    ] = None
    """How the recording was started."""

    start_time: Optional[str] = None
    """The timestamp of when the recording was started."""

    status: Optional[Literal["processing", "absent", "completed", "deleted"]] = None
    """The status of the recording."""

    subresource_uris: Optional[Dict[str, object]] = None
    """A list of related resources identified by their relative URIs."""

    uri: Optional[str] = None
    """The relative URI for this recording."""


class ConferenceRetrieveRecordingsResponse(BaseModel):
    end: Optional[int] = None
    """The number of the last element on the page, zero-indexed."""

    first_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences/6dc6cc1a-1ba1-4351-86b8-4c22c95cd98f/Recordings.json?page=0&pagesize=20
    """

    next_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences/6dc6cc1a-1ba1-4351-86b8-4c22c95cd98f/Recordings.json?Page=1&PageSize=1&PageToken=MTY4AjgyNDkwNzIxMQ
    """

    page: Optional[int] = None
    """Current page number, zero-indexed."""

    page_size: Optional[int] = None
    """The number of items on the page"""

    recordings: Optional[List[Recording]] = None

    start: Optional[int] = None
    """The number of the first element on the page, zero-indexed."""

    uri: Optional[str] = None
    """The URI of the current page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel
from .number_order_with_phone_numbers import NumberOrderWithPhoneNumbers

__all__ = ["NumberOrderStatusUpdateWebhookEvent", "Data", "Meta"]


class Data(BaseModel):
    id: str
    """Unique identifier for the event"""

    event_type: str
    """The type of event being sent"""

    occurred_at: datetime
    """ISO 8601 timestamp of when the event occurred"""

    payload: NumberOrderWithPhoneNumbers

    record_type: str
    """Type of record"""


class Meta(BaseModel):
    attempt: int
    """Webhook delivery attempt number"""

    delivered_to: str
    """URL where the webhook was delivered"""


class NumberOrderStatusUpdateWebhookEvent(BaseModel):
    data: Data

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .mobile_voice_connection import MobileVoiceConnection

__all__ = ["MobileVoiceConnectionUpdateResponse"]


class MobileVoiceConnectionUpdateResponse(BaseModel):
    data: Optional[MobileVoiceConnection] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ManagedAccountGetAllocatableGlobalOutboundChannelsResponse", "Data"]


class Data(BaseModel):
    allocatable_global_outbound_channels: Optional[int] = None
    """
    The total amount of allocatable global outbound channels available to the
    authenticated manager. Will be 0 if the feature is not enabled for their
    account.
    """

    managed_account_allow_custom_pricing: Optional[bool] = None
    """
    Boolean value that indicates if the managed account is able to have custom
    pricing set for it or not. If false, uses the pricing of the manager account.
    Defaults to false. This value may be changed, but there may be time lag between
    when the value is changed and pricing changes take effect.
    """

    record_type: Optional[str] = None
    """The type of the data contained in this record."""

    total_global_channels_allocated: Optional[int] = None
    """
    The total number of allocatable global outbound channels currently allocated
    across all managed accounts for the authenticated user. This includes any amount
    of channels allocated by default at managed account creation time. Will be 0 if
    the feature is not enabled for their account.
    """


class ManagedAccountGetAllocatableGlobalOutboundChannelsResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .ip_connection import IPConnection

__all__ = ["IPConnectionRetrieveResponse"]


class IPConnectionRetrieveResponse(BaseModel):
    data: Optional[IPConnection] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .brand import (
    BrandResource,
    AsyncBrandResource,
    BrandResourceWithRawResponse,
    AsyncBrandResourceWithRawResponse,
    BrandResourceWithStreamingResponse,
    AsyncBrandResourceWithStreamingResponse,
)
from .external_vetting import (
    ExternalVettingResource,
    AsyncExternalVettingResource,
    ExternalVettingResourceWithRawResponse,
    AsyncExternalVettingResourceWithRawResponse,
    ExternalVettingResourceWithStreamingResponse,
    AsyncExternalVettingResourceWithStreamingResponse,
)

__all__ = [
    "ExternalVettingResource",
    "AsyncExternalVettingResource",
    "ExternalVettingResourceWithRawResponse",
    "AsyncExternalVettingResourceWithRawResponse",
    "ExternalVettingResourceWithStreamingResponse",
    "AsyncExternalVettingResourceWithStreamingResponse",
    "BrandResource",
    "AsyncBrandResource",
    "BrandResourceWithRawResponse",
    "AsyncBrandResourceWithRawResponse",
    "BrandResourceWithStreamingResponse",
    "AsyncBrandResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MobilePhoneNumber", "CallForwarding", "CallRecording", "CnamListing", "Inbound", "Outbound"]


class CallForwarding(BaseModel):
    call_forwarding_enabled: Optional[bool] = None

    forwarding_type: Optional[Literal["always", "on-failure"]] = None

    forwards_to: Optional[str] = None


class CallRecording(BaseModel):
    inbound_call_recording_channels: Optional[Literal["single", "dual"]] = None

    inbound_call_recording_enabled: Optional[bool] = None

    inbound_call_recording_format: Optional[Literal["wav", "mp3"]] = None


class CnamListing(BaseModel):
    cnam_listing_details: Optional[str] = None

    cnam_listing_enabled: Optional[bool] = None


class Inbound(BaseModel):
    interception_app_id: Optional[str] = None
    """The ID of the app that will intercept inbound calls."""

    interception_app_name: Optional[str] = None
    """The name of the app that will intercept inbound calls."""


class Outbound(BaseModel):
    interception_app_id: Optional[str] = None
    """The ID of the app that will intercept outbound calls."""

    interception_app_name: Optional[str] = None
    """The name of the app that will intercept outbound calls."""


class MobilePhoneNumber(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    call_forwarding: Optional[CallForwarding] = None

    call_recording: Optional[CallRecording] = None

    caller_id_name_enabled: Optional[bool] = None
    """Indicates if caller ID name is enabled."""

    cnam_listing: Optional[CnamListing] = None

    connection_id: Optional[str] = None
    """The ID of the connection associated with this number."""

    connection_name: Optional[str] = None
    """The name of the connection."""

    connection_type: Optional[str] = None
    """The type of the connection."""

    country_iso_alpha2: Optional[str] = None
    """The ISO 3166-1 alpha-2 country code of the number."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    customer_reference: Optional[str] = None
    """A customer reference string."""

    inbound: Optional[Inbound] = None

    inbound_call_screening: Optional[Literal["disabled", "reject_calls", "flag_calls"]] = None
    """The inbound call screening setting."""

    mobile_voice_enabled: Optional[bool] = None
    """Indicates if mobile voice is enabled."""

    noise_suppression: Optional[Literal["inbound", "outbound", "both", "disabled"]] = None
    """The noise suppression setting."""

    outbound: Optional[Outbound] = None

    phone_number: Optional[str] = None
    """The +E.164-formatted phone number."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    sim_card_id: Optional[str] = None
    """The ID of the SIM card associated with this number."""

    status: Optional[str] = None
    """The status of the phone number."""

    tags: Optional[List[str]] = None
    """A list of tags associated with the number."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["OutboundVoiceProfileListParams", "Filter", "FilterName"]


class OutboundVoiceProfileListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[name][contains]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal[
        "enabled",
        "-enabled",
        "created_at",
        "-created_at",
        "name",
        "-name",
        "service_plan",
        "-service_plan",
        "traffic_type",
        "-traffic_type",
        "usage_payment_method",
        "-usage_payment_method",
    ]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code>-</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>name</code>: sorts the result by the
        <code>name</code> field in ascending order.
      </li>

      <li>
        <code>-name</code>: sorts the result by the
        <code>name</code> field in descending order.
      </li>
    </ul> <br/>
    """


class FilterName(TypedDict, total=False):
    """Name filtering operations"""

    contains: str
    """Optional filter on outbound voice profile name."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[name][contains]
    """

    name: FilterName
    """Name filtering operations"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    NotificationSetting,
    NotificationSettingCreateResponse,
    NotificationSettingDeleteResponse,
    NotificationSettingRetrieveResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNotificationSettings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.create()
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.create(
            notification_channel_id="12455643-3cf1-4683-ad23-1cd32f7d5e0a",
            notification_event_condition_id="70c7c5cb-dce2-4124-accb-870d39dbe852",
            notification_profile_id="12455643-3cf1-4683-ad23-1cd32f7d5e0a",
            parameters=[
                {
                    "name": "phone_number",
                    "value": "+13125550000",
                }
            ],
        )
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.notification_settings.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = response.parse()
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.notification_settings.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = response.parse()
            assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.notification_settings.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = response.parse()
        assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.notification_settings.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = response.parse()
            assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.notification_settings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.list()
        assert_matches_type(SyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.list(
            filter={
                "associated_record_type": {"eq": "phone_number"},
                "channel_type_id": {"eq": "webhook"},
                "notification_channel": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "notification_event_condition_id": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "notification_profile_id": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "status": {"eq": "enable-received"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.notification_settings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.notification_settings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        notification_setting = client.notification_settings.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.notification_settings.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = response.parse()
        assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.notification_settings.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = response.parse()
            assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.notification_settings.with_raw_response.delete(
                "",
            )


class TestAsyncNotificationSettings:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.create()
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.create(
            notification_channel_id="12455643-3cf1-4683-ad23-1cd32f7d5e0a",
            notification_event_condition_id="70c7c5cb-dce2-4124-accb-870d39dbe852",
            notification_profile_id="12455643-3cf1-4683-ad23-1cd32f7d5e0a",
            parameters=[
                {
                    "name": "phone_number",
                    "value": "+13125550000",
                }
            ],
        )
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.notification_settings.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = await response.parse()
        assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.notification_settings.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = await response.parse()
            assert_matches_type(NotificationSettingCreateResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.notification_settings.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = await response.parse()
        assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.notification_settings.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = await response.parse()
            assert_matches_type(NotificationSettingRetrieveResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.notification_settings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.list()
        assert_matches_type(AsyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.list(
            filter={
                "associated_record_type": {"eq": "phone_number"},
                "channel_type_id": {"eq": "webhook"},
                "notification_channel": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "notification_event_condition_id": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "notification_profile_id": {"eq": "12455643-3cf1-4683-ad23-1cd32f7d5e0a"},
                "status": {"eq": "enable-received"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.notification_settings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.notification_settings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[NotificationSetting], notification_setting, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        notification_setting = await async_client.notification_settings.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.notification_settings.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        notification_setting = await response.parse()
        assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.notification_settings.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            notification_setting = await response.parse()
            assert_matches_type(NotificationSettingDeleteResponse, notification_setting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.notification_settings.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["ExportPortoutsCsvReportParam", "Filters"]


class Filters(TypedDict, total=False):
    """The filters to apply to the export port-out CSV report."""

    created_at_gt: Annotated[Union[str, datetime], PropertyInfo(alias="created_at__gt", format="iso8601")]
    """The date and time the port-out was created after."""

    created_at_lt: Annotated[Union[str, datetime], PropertyInfo(alias="created_at__lt", format="iso8601")]
    """The date and time the port-out was created before."""

    customer_reference_in: Annotated[SequenceNotStr[str], PropertyInfo(alias="customer_reference__in")]
    """The customer reference of the port-outs to include in the report."""

    end_user_name: str
    """The end user name of the port-outs to include in the report."""

    phone_numbers_overlaps: Annotated[SequenceNotStr[str], PropertyInfo(alias="phone_numbers__overlaps")]
    """A list of phone numbers that the port-outs phone numbers must overlap with."""

    status_in: Annotated[
        List[Literal["pending", "authorized", "ported", "rejected", "rejected-pending", "canceled"]],
        PropertyInfo(alias="status__in"),
    ]
    """The status of the port-outs to include in the report."""


class ExportPortoutsCsvReportParam(TypedDict, total=False):
    """The parameters for generating a port-outs CSV report."""

    filters: Required[Filters]
    """The filters to apply to the export port-out CSV report."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .hangup_tool_param import HangupToolParam
from .webhook_tool_param import WebhookToolParam
from .transfer_tool_param import TransferToolParam
from .retrieval_tool_param import RetrievalToolParam

__all__ = [
    "AssistantToolParam",
    "Handoff",
    "HandoffHandoff",
    "HandoffHandoffAIAssistant",
    "Refer",
    "ReferRefer",
    "ReferReferTarget",
    "ReferReferCustomHeader",
    "ReferReferSipHeader",
    "SendDtmf",
    "SendMessage",
]


class HandoffHandoffAIAssistant(TypedDict, total=False):
    id: Required[str]
    """The ID of the assistant to hand off to."""

    name: Required[str]
    """Helpful name for giving context on when to handoff to the assistant."""


class HandoffHandoff(TypedDict, total=False):
    ai_assistants: Required[Iterable[HandoffHandoffAIAssistant]]
    """List of possible assistants that can receive a handoff."""

    voice_mode: Literal["unified", "distinct"]
    """
    With the unified voice mode all assistants share the same voice, making the
    handoff transparent to the user. With the distinct voice mode all assistants
    retain their voice configuration, providing the experience of a conference call
    with a team of assistants.
    """


class Handoff(TypedDict, total=False):
    """
    The handoff tool allows the assistant to hand off control of the conversation to another AI assistant. By default, this will happen transparently to the end user.
    """

    handoff: Required[HandoffHandoff]

    type: Required[Literal["handoff"]]


class ReferReferTarget(TypedDict, total=False):
    name: Required[str]
    """The name of the target."""

    sip_address: Required[str]
    """The SIP URI to which the call will be referred."""

    sip_auth_password: str
    """SIP Authentication password used for SIP challenges."""

    sip_auth_username: str
    """SIP Authentication username used for SIP challenges."""


class ReferReferCustomHeader(TypedDict, total=False):
    name: str

    value: str
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class ReferReferSipHeader(TypedDict, total=False):
    name: Literal["User-to-User", "Diversion"]

    value: str
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class ReferRefer(TypedDict, total=False):
    targets: Required[Iterable[ReferReferTarget]]
    """The different possible targets of the SIP refer.

    The assistant will be able to choose one of the targets to refer the call to.
    """

    custom_headers: Iterable[ReferReferCustomHeader]
    """Custom headers to be added to the SIP REFER."""

    sip_headers: Iterable[ReferReferSipHeader]
    """SIP headers to be added to the SIP REFER.

    Currently only User-to-User and Diversion headers are supported.
    """


class Refer(TypedDict, total=False):
    refer: Required[ReferRefer]

    type: Required[Literal["refer"]]


class SendDtmf(TypedDict, total=False):
    send_dtmf: Required[Dict[str, object]]

    type: Required[Literal["send_dtmf"]]


class SendMessage(TypedDict, total=False):
    """
    The send_message tool allows the assistant to send SMS or MMS messages to the end user. The 'to' and 'from' addresses are automatically determined from the conversation context, and the message text is generated by the assistant.
    """

    send_message: Required[Dict[str, object]]

    type: Required[Literal["send_message"]]


AssistantToolParam: TypeAlias = Union[
    WebhookToolParam, RetrievalToolParam, Handoff, HangupToolParam, TransferToolParam, Refer, SendDtmf, SendMessage
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.messaging_10dlc.brand import (
    ExternalVettingListResponse,
    ExternalVettingOrderResponse,
    ExternalVettingImportsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExternalVetting:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        external_vetting = client.messaging_10dlc.brand.external_vetting.list(
            "brandId",
        )
        assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.brand.external_vetting.with_raw_response.list(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = response.parse()
        assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.messaging_10dlc.brand.external_vetting.with_streaming_response.list(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = response.parse()
            assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.messaging_10dlc.brand.external_vetting.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_imports(self, client: Telnyx) -> None:
        external_vetting = client.messaging_10dlc.brand.external_vetting.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        )
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_imports_with_all_params(self, client: Telnyx) -> None:
        external_vetting = client.messaging_10dlc.brand.external_vetting.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
            vetting_token="vettingToken",
        )
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_imports(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.brand.external_vetting.with_raw_response.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = response.parse()
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_imports(self, client: Telnyx) -> None:
        with client.messaging_10dlc.brand.external_vetting.with_streaming_response.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = response.parse()
            assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_imports(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.messaging_10dlc.brand.external_vetting.with_raw_response.imports(
                brand_id="",
                evp_id="evpId",
                vetting_id="vettingId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_order(self, client: Telnyx) -> None:
        external_vetting = client.messaging_10dlc.brand.external_vetting.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        )
        assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_order(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.brand.external_vetting.with_raw_response.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = response.parse()
        assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_order(self, client: Telnyx) -> None:
        with client.messaging_10dlc.brand.external_vetting.with_streaming_response.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = response.parse()
            assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_order(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.messaging_10dlc.brand.external_vetting.with_raw_response.order(
                brand_id="",
                evp_id="evpId",
                vetting_class="vettingClass",
            )


class TestAsyncExternalVetting:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        external_vetting = await async_client.messaging_10dlc.brand.external_vetting.list(
            "brandId",
        )
        assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.list(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = await response.parse()
        assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.brand.external_vetting.with_streaming_response.list(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = await response.parse()
            assert_matches_type(ExternalVettingListResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_imports(self, async_client: AsyncTelnyx) -> None:
        external_vetting = await async_client.messaging_10dlc.brand.external_vetting.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        )
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_imports_with_all_params(self, async_client: AsyncTelnyx) -> None:
        external_vetting = await async_client.messaging_10dlc.brand.external_vetting.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
            vetting_token="vettingToken",
        )
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_imports(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = await response.parse()
        assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_imports(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.brand.external_vetting.with_streaming_response.imports(
            brand_id="brandId",
            evp_id="evpId",
            vetting_id="vettingId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = await response.parse()
            assert_matches_type(ExternalVettingImportsResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_imports(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.imports(
                brand_id="",
                evp_id="evpId",
                vetting_id="vettingId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_order(self, async_client: AsyncTelnyx) -> None:
        external_vetting = await async_client.messaging_10dlc.brand.external_vetting.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        )
        assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_order(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_vetting = await response.parse()
        assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_order(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.brand.external_vetting.with_streaming_response.order(
            brand_id="brandId",
            evp_id="evpId",
            vetting_class="vettingClass",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_vetting = await response.parse()
            assert_matches_type(ExternalVettingOrderResponse, external_vetting, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_order(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.messaging_10dlc.brand.external_vetting.with_raw_response.order(
                brand_id="",
                evp_id="evpId",
                vetting_class="vettingClass",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["InferenceEmbeddingTransferToolParamsParam", "Target", "CustomHeader"]


class Target(TypedDict, total=False):
    name: str
    """The name of the target."""

    to: str
    """The destination number or SIP URI of the call."""


class CustomHeader(TypedDict, total=False):
    name: str

    value: str
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


_InferenceEmbeddingTransferToolParamsParamReservedKeywords = TypedDict(
    "_InferenceEmbeddingTransferToolParamsParamReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class InferenceEmbeddingTransferToolParamsParam(
    _InferenceEmbeddingTransferToolParamsParamReservedKeywords, total=False
):
    targets: Required[Iterable[Target]]
    """The different possible targets of the transfer.

    The assistant will be able to choose one of the targets to transfer the call to.
    """

    custom_headers: Iterable[CustomHeader]
    """Custom headers to be added to the SIP INVITE for the transfer command."""

    warm_transfer_instructions: str
    """
    Natural language instructions for your agent for how to provide context for the
    transfer recipient.
    """
